package io.resourcepool.ssdp.client.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.*;

/**
 * Utils used by the SsdpClient.
 *
 * @author Loïc Ortola on 05/08/2017
 */
public abstract class Utils {
  private static Logger LOG = LoggerFactory.getLogger(Utils.class);
  /**
   * Creates a list of viable network interfaces for Multicast.
   *
   * @throws SocketException if something bad happens
   * @return list of interfaces
   */
  public static List<NetworkInterface> getMulticastInterfaces() throws SocketException {
    Set<NetworkInterface> viableInterfaces =new HashSet<>();
    LOG.info("getMulticastInterfaces");
    Enumeration e = NetworkInterface.getNetworkInterfaces();
    LOG.info("getMulticastInterfaces getNetworkInterfaces");
    while (e.hasMoreElements()) {
      NetworkInterface n = (NetworkInterface) e.nextElement();
      // Added due to Android security requirements
      // @see @jeremiabitterfeld https://github.com/resourcepool/ssdp-client/issues/28#issuecomment-1000477607
//      if(!n.supportsMulticast() ) {
//        LOG.info("{}  /  {}not supportsMulticast",n.getName() ,n.getDisplayName());
//        continue;
//      }
      Enumeration ee = n.getInetAddresses();
      while (ee.hasMoreElements()) {
        InetAddress i = (InetAddress) ee.nextElement();
        if (i instanceof Inet4Address) {
          if  (!"lo".equalsIgnoreCase(n.getName()) &&!i.isLoopbackAddress() && !i.isMulticastAddress()){
            viableInterfaces.add(n);
            LOG.info("get interface ip:{},{},{},", n.getName(),n.getDisplayName() ,i.getHostAddress());
            break;
          }
//        if (i.isSiteLocalAddress() && !i.isAnyLocalAddress() && !i.isLinkLocalAddress()
//            && ) {

        }
      }
    }
    List<NetworkInterface> interfaces=new ArrayList<>();
    viableInterfaces.forEach(interfaces::add);
    LOG.info("Utils: getMulticastInterfaces: interfaces: {}", interfaces);
    return interfaces;
  }
}
